import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { connect } from 'dva';
import { Spin, Badge, Table, Card, Form, Input, Icon, Button } from 'antd';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import WareSelectSortModal from './modal';
import { Codes } from '../../../utils/config';

@Form.create()
@connect(state => ({
  lessonDetail: state.lessonDetail,
  ware: state.ware,
}))
export default class LessonDetail extends PureComponent {
  // 展示模态框 - 加载第一页数据
  handleModalShow = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'lessonDetail/showModal' });
    dispatch({ type: 'ware/query', payload: { pageNo: 1, pageSize: 10 } });
  }

  // 取消/关闭 - 隐藏模态框
  handleModalCancel = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'lessonDetail/hideModal' });
  }

  // 提交 - 保存选择和排序完的数据到model中
  handleModalOk = (data) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'lessonDetail/saveSortResult',
      payload: { wareList: data }
    });
  }

  // 搜索
  handleModalSearch = (data) => {
    const { dispatch } = this.props;
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;

    dispatch({
      type: 'ware/query',
      payload: { ...newData, pageNo: 1, pageSize: 10 },
    });
  }

  // 翻页 - 资源列表
  handleModalTableOnChange = (pagination, filterArgs, filters) => {
    const { dispatch } = this.props;
    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');
    const tableFilters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});

    const data = { ...newFilters, ...tableFilters, pageNo: pagination.current, pageSize: pagination.pageSize };
    Object.keys(data).map(key => data[key] ? null : delete data[key]);
    dispatch({ type: 'ware/query', payload: data });
  }

  handlePageSubmit = (e) => {
    e.preventDefault()
    const {
      dispatch,
      form: {
        validateFields,
        getFieldsValue,
        resetFields
      },
      lessonDetail: {
        operType,
        currentItem,
        filters,
      }
    } = this.props;
    validateFields((errors) => {
      if (errors) return;

      const { wareList, gmtCreated, gmtModified, ...rest } = currentItem;
      const data = {
        ...rest,
        ...getFieldsValue(),
      };

      // 课件数组id list
      const idList = (wareList || []).map(item => item.id);
      data.wareList = idList;

      // 新建课，status设为normal
      if (operType == 'create') {
        data.status = Codes.CODE_NORMAL;
      }

      dispatch({
        type: `lessonDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch(
            routerRedux.push({
              pathname: '/basic-product/lesson',
              search: queryString.stringify(filters),
            })
          );
          resetFields();
        }
      });
    });
  }

  handlePageCancel = () => {
    const { dispatch, lessonDetail: { filters } } = this.props;
    dispatch({ type: 'lessonDetail/initState' });
    dispatch(
      routerRedux.push({
        pathname: '/basic-product/lesson',
        search: queryString.stringify(filters),
      })
    );
  }

  render() {
    const { dispatch, form: { getFieldDecorator }, lessonDetail, ware } = this.props;
    const { itemLoading, currentItem, filters, modalVisible } = lessonDetail;
    const { wareList, name, code, digest } = currentItem;
    const { list, listLoading, pagination } = ware;

    // 待选表格去掉分页的跳转及变换页码
    if (pagination) {
      delete pagination.showQuickJumper;
      delete pagination.showSizeChanger;
    }

    const subTableColumns = [{
      title: '课件编号',
      dataIndex: 'code',
      key: 'code',
      width: '50%',
    },{
      title: '课件名称',
      dataIndex: 'name',
      key: 'name',
      width: '50%',
    }];

    const formItemLayout = {
      labelCol: {
        span: 7,
      },
      wrapperCol: {
        span: 12,
      },
    };
    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card>
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="课编号：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('code', {
                  rules: [{ required: true, type: 'string', message: "编号为必填项!" }],
                  initialValue: code,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课名称：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: "名称为必填项!" }],
                  initialValue: name,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课简述：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('digest', {
                  initialValue: digest,
                })(<Input.TextArea />)}
              </Form.Item>
              <Form.Item label="选择课件" {...formItemLayout}>
                <Button onClick={this.handleModalShow} type="primary" size="small" icon="edit">编辑</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该课下不包含任何内容，请选择相关课件!</span>
                  }}
                  dataSource={wareList}
                  columns={subTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item {...submitFormLayout} style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            <WareSelectSortModal
              rowKeyName="id"
              modalVisible={modalVisible}
              style={{ top: 20 }}
              width={600}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
              onSearch={this.handleModalSearch}
              selTableData={wareList || []}
              fsTableDataSource={list}
              fsTableLoading={listLoading}
              fsTablePagination={pagination}
              fsTableOnChange={this.handleModalTableOnChange}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
