import React, { Component } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { Card, Modal, Button, Table } from 'antd';
import moment from 'moment';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import FooterToolbar from '../../../components/FooterToolbar';
import { Codes, orderStatuses, productType, orderSplitStatus } from '../../../utils/config';
import { provCodeToName } from '../../../utils/city';

@connect(state => ({ orderDetail: state.orderDetail }))
export default class OrderProfile extends Component {
  /**
   * 退出页面
   */
  handlePageCancel = () => {
    const { filters, dispatch } = this.props;
    dispatch(routerRedux.push({
      pathname: '/trade/order',
      search: queryString.stringify(filters),
    }));
  }

  /**
   * 确认支付
   */
  confirmPay = () => {
    const { dispatch, orderDetail } = this.props;
    const { currentItem, filters } = orderDetail;
    const { id } = currentItem;
    Modal.confirm({
      okText: '确认支付',
      cancelText: '取消支付',
      title: '确定支付该订单?',
      onOk: () => {
        dispatch({
          type: 'orderDetail/orderPay',
          payload: { id },
          callback: () => dispatch(routerRedux.push({
            pathname: `/trade/order/profile/${id}`,
            search: queryString.stringify(filters),
          })),
        });
      },
    });
  }

  /**
   * 渲染底部操作按钮
   */
  renderFooter = (status) => {
    switch (status) {
      case Codes.CODE_UNPAID:
        return (
          <FooterToolbar>
            <Button onClick={this.handlePageCancel}>返回订单列表</Button>
            <Button onClick={this.confirmPay} type="primary">立即支付</Button>
          </FooterToolbar>
        );
      default:
        return (
          <FooterToolbar>
            <Button onClick={this.handlePageCancel}>返回订单列表</Button>
          </FooterToolbar>
        );
    }
  }

  render() {
    const { orderDetail } = this.props;
    const { currentItem } = orderDetail;
    const {
      id,
      gmtCreated,
      userCode,
      provinceCode,
      cityName,
      zoneName,
      classroomName,
      orderStatus,
      splitStatus,
      merchantName,
      originPrice,
      finalPrice,
      adjustPrice,
      name,
      mobile,
      address,
      note,
      goods,
    } = currentItem;

    // simply table columns
    const simplyTableColumns = [{
      title: 'field',
      dataIndex: 'field',
      key: 1,
      width: '20%',
    }, {
      title: 'value',
      dataIndex: 'value',
      key: 2,
      width: '80%',
    }];
    // 订单信息 - table data
    const orderInfoTableDatas = [{
      field: '订单编号',
      value: id,
      key: 1,
    }, {
      field: '下单时间',
      value: moment(gmtCreated).format('YYYY-MM-DD HH:mm:ss'),
      key: 2,
    }, {
      field: '订单状态',
      value: orderStatuses[orderStatus],
      key: 3,
    }, {
      field: '拆分状态',
      value: orderSplitStatus[splitStatus],
      key: 4,
    }, {
      field: '拆分原因',
      value: '订单中包含实体物品或物品分属不同商家',
      key: 5,
    }];
    // 收货人信息 - table data
    const consumerInfoTableData = [{
      field: '购买终端编号',
      value: userCode,
      key: 1,
    }, {
      field: '购买终端名称',
      value: `${classroomName}`,
      key: 2,
    }, {
      field: '终端所属校区',
      value: `${provCodeToName(provinceCode)}-${cityName}-${zoneName}`,
      key: 3,
    }, {
      field: '终端所属渠道',
      value: merchantName,
      key: 4,
    }, {
      field: '收货人姓名',
      value: name,
      key: 5,
    }, {
      field: '收货人电话',
      value: mobile,
      key: 6,
    }, {
      field: '收货地址',
      value: address,
      key: 7,
    }];
    // 商品清单
    const goodsTableColumns = [{
      title: '商品编号',
      dataIndex: 'code',
      key: 1,
      width: '20%',
    }, {
      title: '商品名称',
      dataIndex: 'name',
      key: 2,
      width: '20%',
    }, {
      title: '商品类型',
      dataIndex: 'type',
      render: text => productType[text],
      key: 3,
      width: '15%',
    }, {
      title: '商品售价(元)',
      dataIndex: 'merchantPrice',
      key: 4,
      width: '20%',
    }, {
      title: '商品数量',
      dataIndex: 'quantity',
      key: 5,
      width: '10%',
    }, {
      title: '操作',
      dataIndex: 'operation',
      render: () => <a>查看详情</a>,
      key: 6,
      width: '15%',
    }];
    // 结算信息 - table data
    const strs = [];
    (goods || []).forEach(item => strs.push(`${item.merchantPrice}(元) * ${item.quantity}`));
    const accountTableDatas = [{
      field: '商品总价',
      value: () => `${strs.join(' + ')} = ${originPrice}(元)`,
      key: 1,
    }, {
      field: '支付金额',
      value: () => (
        <span>
          {`总价：${originPrice}(元) - 商品优惠：${adjustPrice}(元) = 订单总金额：`}
          <strong style={{ color: 'red' }}>{`${finalPrice}(元)`}</strong>
        </span>),
      key: 2,
    }];

    return (
      <PageHeaderLayout title="订单详情">
        <Card title="订单信息" bordered={false} style={{ marginBottom: 20 }}>
          <Table
            size="small"
            columns={simplyTableColumns}
            dataSource={orderInfoTableDatas}
            bordered
            pagination={false}
            showHeader={false}
          />
        </Card>
        <Card title="商品清单" bordered={false} style={{ marginBottom: 20 }}>
          <Table
            size="small"
            rowKey={record => record.id}
            columns={goodsTableColumns}
            dataSource={goods || []}
            bordered
            pagination={false}
          />
        </Card>
        <Card title="收货人信息" bordered={false} style={{ marginBottom: 20 }}>
          <Table
            size="small"
            columns={simplyTableColumns}
            dataSource={consumerInfoTableData}
            bordered
            pagination={false}
            showHeader={false}
          />
        </Card>
        <Card title="订单备注" bordered={false} style={{ marginBottom: 20 }}>
          {note || '无'}
        </Card>
        <Card title="结算信息" bordered={false}>
          <Table
            size="small"
            columns={simplyTableColumns}
            dataSource={accountTableDatas}
            bordered
            pagination={false}
            showHeader={false}
          />
        </Card>
        {this.renderFooter(orderStatus)}
      </PageHeaderLayout>
    );
  }
}
