import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { connect } from 'dva';
import { Spin, Badge, Table, Card, Form, Input, Icon, Button, Select } from 'antd';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import TagSortModal from './modal';
import { Codes, statuses, tagType } from '../../../utils/config';

@Form.create()
@connect(state => ({
  groupDetail: state.groupDetail,
  merchant: state.merchant,
}))
export default class GroupDetail extends PureComponent {
  // 组件一挂载完成，立即请求1000条厂商数据
  // TODO 有弊端，待后期优化
  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'merchant/query',
      payload: {
        pageNo: 1,
        pageSize: 1000,
      }
    });
  }

  handleModalShow = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'groupDetail/showModal' });
  }

  handleModalCancel = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'groupDetail/hideModal' });
  }

  handleModalOk = (data) => {
    console.log('hahaha');
    const { dispatch } = this.props;
    dispatch({
      type: 'groupDetail/saveSortResult',
      payload: { tagList: data }
    });
  }

  handlePageSubmit = (e) => {
    e.preventDefault()
    const {
      dispatch,
      form: {
        validateFields,
        getFieldsValue,
      },
      groupDetail: {
        operType,
        currentItem,
        filters,
      }
    } = this.props;
    validateFields((errors) => {
      if (errors) return;

      let data = {};
      if (operType == 'create') {
        data = {
          ...getFieldsValue(),
          status: Codes.CODE_NORMAL
        };
      } else if (operType == 'update') {
        const { id, merchantId, status, tagList, name, code } = currentItem;
        data = {
          id,
          merchantId,
          status,
          name,
          code,
          tagList: (tagList || []).map(item => item.id),
          ...getFieldsValue(),
        };
      }

      dispatch({
        type: `groupDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch({ type: 'groupDetail/initState' });
          dispatch(
            routerRedux.push({
              pathname: '/tag/tagGroup',
              search: queryString.stringify(filters),
            })
          );
        }
      })
    });
  }

  handlePageCancel = () => {
    const { dispatch, groupDetail: { filters } } = this.props;
    dispatch({ type: 'groupDetail/initState' });
    dispatch(
      routerRedux.push({
        pathname: '/tag/tagGroup',
        search: queryString.stringify(filters),
      })
    );
  }

  render() {
    const { dispatch, form, groupDetail, merchant } = this.props;
    const { itemLoading, currentItem, filters, modalVisible, operType } = groupDetail;
    const { tagList, name, code, merchantId } = currentItem;
    const { getFieldDecorator } = form;
    const { list } = merchant;

    const tagTableColumns = [{
      title: '位置',
      dataIndex: 'sort',
      key: 'sort',
      render: (text, record, index) => index + 1,
    },{
      title: '标签名称',
      dataIndex: 'name',
      key: 'name',
    },{
      title: '标签状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = {[Codes.CODE_NORMAL]: 'success', [Codes.CODE_DELETE]: 'error'};
        return (<Badge status={statusMap[record.status]} text={statuses[record.status]} />);
      },
    }];

    const formItemLayout = {
      labelCol: {
        span: 7,
      },
      wrapperCol: {
        span: 12,
      },
    };
    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card>
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="标签组编号：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('code', {
                  rules: [{ required: true, type: 'string', message: "编号为必填项!" }],
                  initialValue: code,
                })(<Input placeholder="请输入" />)}
              </Form.Item>
              <Form.Item label="标签组名称：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: "名称为必填项!" }],
                  initialValue: name,
                })(<Input placeholder="请输入" />)}
              </Form.Item>
              <Form.Item label="渠道名称" hasFeedback {...formItemLayout}>
                {getFieldDecorator('merchantId', {
                  rules: [{ required: true, type: 'string', message: "渠道为必选项!" }],
                  initialValue: merchantId,
                })(
                  <Select
                    showSearch
                    allowClear
                    placeholder="请输入渠道编号或者名称进行筛选"
                    optionFilterProp="children"
                    disabled={operType == 'update' ? true : false}
                    filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                  >
                    {list.map(item => <Select.Option value={item.id} key={item.id}>{`${item.code}/${item.name}`}</Select.Option>)}
                  </Select>
                )}
              </Form.Item>
              <Form.Item label="标签排序" {...formItemLayout}>
                <Button onClick={this.handleModalShow} disabled={!tagList || !tagList.length} type="primary" size="small" icon="edit">排序</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>该标签组下不包含任何标签，无法排序，请先去关联标签吧!</span>
                  }}
                  dataSource={tagList || []}
                  columns={tagTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item {...submitFormLayout} style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            <TagSortModal
              tagList={tagList || []}
              rowKeyName="id"
              modalVisible={modalVisible}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
