import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import queryString from 'query-string';
import { connect } from 'dva';
import { Spin, Popover, Table, Card, Form, Input, Icon, Button } from 'antd';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import ResourceSelectSortModal from './modal';
import { Codes, ossHost, resourceType, pageSize } from '../../../utils/config';

@Form.create()
@connect(state => ({
  wareDetail: state.wareDetail,
  resource: state.resource,
}))
export default class WareDetail extends PureComponent {
  state = { resType: [String(Codes.CODE_IMAGE)] }; // 资源类型[默认为图片资源]

  // 展示模态框 - 加载第一页数据
  handleModalShow = () => {
    const { dispatch } = this.props;
    const { resType } = this.state;
    dispatch({ type: 'wareDetail/showModal' });
    dispatch({
      type: 'resource/query',
      payload: {
        pageNo: 1,
        pageSize,
        type: resType[0],
      }
    });
  }

  // 取消/关闭 - 隐藏模态框
  handleModalCancel = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'wareDetail/hideModal' });
  }

  // 提交 - 保存选择和排序完的数据到model中
  handleModalOk = (data) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'wareDetail/saveSortResult',
      payload: { resourceList: data }
    });
  }

  // 搜索
  handleModalSearch = (data) => {
    const { dispatch } = this.props;
    const { resType } = this.state;
    const newData = { ...data };
    if (newData.keyword) {
      newData[newData.field] = newData.keyword;
    }
    delete newData.field;
    delete newData.keyword;

    dispatch({
      type: 'resource/query',
      payload: {
        ...newData,
        type: resType[0],
        pageNo: 1,
        pageSize,
      },
    });
  }

  // 翻页 - 资源列表
  handleModalTableOnChange = (pagination, filterArgs, filters) => {
    // 待选资源列表中资源类型过滤选项
    const { type } = filterArgs;
    if (Array.isArray(type) && type.length) {
      this.setState({ resType: type.map(item => String(item)) });
    } else {
      this.setState({ resType: [String(Codes.CODE_IMAGE)] });
      filterArgs.type = [String(Codes.CODE_IMAGE)];
    }

    // 整理要搜索的内容及过滤项
    const { dispatch } = this.props;
    const newFilters = { ...filters };
    if (newFilters.keyword) {
      newFilters[newFilters.field] = newFilters.keyword;
    }
    delete newFilters.field;
    delete newFilters.keyword;

    const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');
    const tableFilters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});
    const data = { ...newFilters, ...tableFilters, pageNo: pagination.current, pageSize: pagination.pageSize };
    Object.keys(data).map(key => data[key] ? null : delete data[key]);
    dispatch({ type: 'resource/query', payload: data });
  }

  handlePageSubmit = (e) => {
    e.preventDefault()
    const {
      dispatch,
      form: {
        validateFields,
        getFieldsValue,
        resetFields
      },
      wareDetail: {
        operType,
        currentItem,
        filters,
      }
    } = this.props;
    validateFields((errors) => {
      if (errors) return;

      // 删除不必要字段或者需处理字段(resourceList)
      const { gmtCreated, gmtModified, resourceList, ...rest } = currentItem;

      // 获取全部表单数据及接口数据
      const data = {
        ...rest,
        ...getFieldsValue(),
      };

      // resourceList为idList
      data.resourceList = (resourceList || []).map(item => item.id);

      // 创建课件状态为normal
      if (operType == 'create') {
        data.status = Codes.CODE_NORMAL;
      }

      dispatch({
        type: `wareDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch(
            routerRedux.push({
              pathname: '/basic-product/ware',
              search: queryString.stringify(filters),
            })
          );
          resetFields();
        }
      });
    });
  }

  handlePageCancel = () => {
    const { dispatch, wareDetail: { filters } } = this.props;
    dispatch({ type: 'wareDetail/initState' });
    dispatch(
      routerRedux.push({
        pathname: '/basic-product/ware',
        search: queryString.stringify(filters),
      })
    );
  }

  render() {
    const { dispatch, wareDetail, resource, form } = this.props;
    const { resType } = this.state;
    const { getFieldDecorator } = form;
    const { itemLoading, currentItem, filters, modalVisible } = wareDetail;
    const { resourceList, name, code, digest, category } = currentItem;
    const { list, listLoading, pagination } = resource;

    // 待选表格去掉分页的跳转及变换页码
    if (pagination) {
      delete pagination.showQuickJumper;
      delete pagination.showSizeChanger;
    }

    const subTableColumns = [{
      title: '缩略图',
      dataIndex: 'url',
      key: 'url',
      render: (text, record) => (
        record.type !== Codes.CODE_IMAGE ? <Icon type="video-camera" style={{ fontSize: 40}} /> :
          <Popover
            content={<img alt="" src={`${ossHost}/${record.path}`} width={350} />}
            title={record.name}
          >
            <img alt="" src={`${ossHost}/${record.path}`} width={70} />
          </Popover>
      ),
    },{
      title: '资源编号',
      dataIndex: 'code',
      key: 'code',
    },{
      title: '资源名称',
      dataIndex: 'name',
      key: 'name',
    },{
      title: '资源类型',
      dataIndex: 'type',
      key: 'type',
      render: (text, record) => (resourceType[record.type]),
    }];

    const formItemLayout = {
      labelCol: {
        span: 7,
      },
      wrapperCol: {
        span: 12,
      },
    };
    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card>
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="课件编号：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('code', {
                  rules: [{ required: true, type: 'string', message: "编号为必填项!" }],
                  initialValue: code,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课件名称：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: "名称为必填项!" }],
                  initialValue: name,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课件分类：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('category', {
                  initialValue: category,
                })(<Input />)}
              </Form.Item>
              <Form.Item label="课件简述：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('digest', {
                  initialValue: digest,
                })(<Input.TextArea />)}
              </Form.Item>
              <Form.Item label="选择资源" {...formItemLayout}>
                <Button onClick={this.handleModalShow} type="primary" size="small" icon="edit">编辑</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该课件下不包含任何内容，请选择图片或视频!</span>
                  }}
                  dataSource={resourceList || []}
                  columns={subTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item {...submitFormLayout} style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            <ResourceSelectSortModal
              rowKeyName="id"
              modalVisible={modalVisible}
              style={{ top: 20 }}
              width={600}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
              onSearch={this.handleModalSearch}
              selTableData={resourceList || []}
              fsTableDataSource={list}
              fsTableLoading={listLoading}
              fsTablePagination={pagination}
              fsFilteredValue={resType}
              fsTableOnChange={this.handleModalTableOnChange}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
